"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.creteWebhookServer = exports.craftWebhookPath = exports.ALLOWED_WEBHOOK_PORTS = void 0;
const fs_1 = __importDefault(require("fs"));
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const utils_1 = require("../utils");
const logger_1 = require("./logger");
const errors_1 = require("../errors");
exports.ALLOWED_WEBHOOK_PORTS = [80, 88, 443, 8443];
function craftWebhookPath(url, token) {
    return `${utils_1.getUrlPathname(url)}${token}`;
}
exports.craftWebhookPath = craftWebhookPath;
const listener = (bot, path) => (request, response) => {
    if (request.url === path && request.method === "POST") {
        const body = [];
        request.on("data", (chunk) => body.push(chunk));
        request.on("end", () => {
            let update;
            try {
                update = utils_1.convertToArray(JSON.parse(body.join()));
            }
            catch (error) {
                bot.logger.error(logger_1.LID.Webhook, { error });
                response.writeHead(415);
                response.end();
                return error;
            }
            bot.logger.debug(logger_1.LID.Webhook, { meta: { update } });
            bot.processTelegramUpdates(update).catch((e) => {
                const error = errors_1.normalizeError(e);
                bot.logger.error(logger_1.LID.Webhook, { error });
                response.writeHead(500);
                return error;
            }).finally(() => {
                response.end();
            });
        });
    }
};
function creteWebhookServer(bot, options) {
    const { serverHost, serverPort } = options;
    const key = options.serverKey && fs_1.default.readFileSync(options.serverKey);
    const cert = options.serverCert && fs_1.default.readFileSync(options.serverCert);
    const isSecure = !!(key && cert);
    const path = craftWebhookPath(options.url, bot.getToken());
    const server = isSecure ?
        https_1.default.createServer({ key, cert }, listener(bot, path)) :
        http_1.default.createServer(listener(bot, path));
    return new Promise((resolve) => {
        server.listen(serverPort, serverHost, () => {
            bot.logger.info(logger_1.LID.Webhook, { meta: { serverHost, serverPort } });
            resolve();
        });
    });
}
exports.creteWebhookServer = creteWebhookServer;
